import matplotlib.pyplot as plt
import numpy as np
import sys

STATE_ABBR = {
    "ALABAMA": "AL", "ALASKA": "AK", "AMERICAN SAMOA": "AS",
    "ARIZONA": "AZ", "ARKANSAS": "AR", "CALIFORNIA": "CA",
    "COLORADO": "CO", "CONNECTICUT": "CT", "DELAWARE": "DE",
    "DISTRICT OF COLUMBIA": "DC", "DODEA": 'DO',
    "FLORIDA": "FL", "GEORGIA": "GA",
    "GUAM": "GU", "HAWAII": "HI", "IDAHO": "ID",
    "ILLINOIS": "IL", "INDIANA": "IN", "IOWA": "IA",
    "KANSAS": "KS", "KENTUCKY": "KY", "LOUISIANA": "LA",
    "MAINE": "ME", "MARYLAND": "MD", "MASSACHUSETTS": "MA",
    "MICHIGAN": "MI", "MINNESOTA": "MN", "MISSISSIPPI": "MS",
    "MISSOURI": "MO", "MONTANA": "MT", "NATIONAL": "ALL", "NEBRASKA": "NE",
    "NEVADA": "NV", "NEW HAMPSHIRE": "NH", "NEW JERSEY": "NJ",
    "NEW MEXICO": "NM", "NEW YORK": "NY", "NORTH CAROLINA": "NC",
    "NORTH DAKOTA": "ND", "NORTHERN MARIANA IS": "MP", "OHIO": "OH",
    "OKLAHOMA": "OK", "OREGON": "OR", "PENNSYLVANIA": "PA",
    "PUERTO RICO": "PR", "RHODE ISLAND": "RI", "SOUTH CAROLINA": "SC",
    "SOUTH DAKOTA": "SD", "TENNESSEE": "TN", "TEXAS": "TX",
    "UTAH": "UT", "VERMONT": "VT", "VIRGINIA": "VA",
    "VIRGIN ISLANDS": "VI", "WASHINGTON": "WA", "WEST VIRGINIA": "WV",
    "WISCONSIN": "WI", "WYOMING": "WY",
}

def main():
    examples = ['simple',
                'multiple_plots',
                'multiple_plots_no_numpy',
                'eighth_grade_reading_scores_2019',
                'eighth_grade_reading_and_math_scores_2019',
                ]
    print("Choose an example:")
    for idx, ex in enumerate(examples):
        print(f'({idx}) {ex}')

    choice = input('What number? ')
    choice_fn = examples[int(choice)]
    plt = eval(f'{choice_fn}()')
    if len(sys.argv) > 1 and sys.argv[1] == '--save':
        plt.savefig(f'{choice_fn}.png')
    plt.show()

def simple():
    fig, ax = plt.subplots()  # Create a figure containing a single axes.
    ax.plot([1, 2, 3, 4], [1, 4, 2, 3])  # Plot some data on the axes.
    return plt


def multiple_plots():
    x = np.linspace(0, 2, 100)
    # numpy array with 100 numbers evenly
    # spaced from 0 to 2. Kind of equivalent to
    # [2 * x / 99 for x in range(100)]
    # except that numpy arrays allow you to do math
    # directly on them

    plt.plot(x, x, label='linear')  # Plot some data on the (implicit) axes.
    plt.plot(x, x ** 2, label='quadratic')  # etc.
    plt.plot(x, x ** 3, label='cubic')
    plt.xlabel('x label')
    plt.ylabel('y label')
    plt.title("Simple Plot")
    plt.legend()
    return plt

def multiple_plots_no_numpy():
    x = [2 * x / 99 for x in range(100)]

    plt.plot(x, x, label='linear')  # Plot some data on the (implicit) axes.
    plt.plot(x, [n ** 2 for n in x], label='quadratic')  # etc.
    plt.plot(x, [n ** 3 for n in x], label='cubic')
    plt.xlabel('x label')
    plt.ylabel('y label')
    plt.title("Simple Plot")
    plt.legend()
    return plt

def eighth_grade_reading_scores_2019():
    fig, ax = plt.subplots()
    with open('states_all.csv') as f:
        lines = f.readlines()[1:]

    lines = [x for x in lines if x.startswith('2019')]
    # state is index 1
    # AVG_READING_8_SCORE is the last column
    x_values = [x.split(',')[1].replace('_', ' ') for x in lines]

    # convert to abbrev
    x_values = [STATE_ABBR[x] for x in x_values]
    y_values = [int(float(x.split(',')[-1].strip())) for x in lines]

    # sort
    all_values = sorted(zip(x_values, y_values), key=lambda x: x[1],
                        reverse=True)

    # extract again
    x_values = [x[0] for x in all_values]
    y_values = [x[1] for x in all_values]

    x_pos = [i for i in range(len(x_values))]

    plt.bar(x_pos, y_values, color='green')
    plt.xlabel("State")
    plt.ylabel("Average 8th Grade Reading Score")
    plt.title("Average 8th Grade Reading Score for U.S. States")

    plt.xticks(x_pos, x_values, rotation=90)
    # scale the y-axis so the bars aren't so huge
    ax.set(ylim=[min(y_values) - 5, max(y_values)])

    return plt


def eighth_grade_reading_and_math_scores_2019():
    fig, ax = plt.subplots()
    with open('states_all.csv') as f:
        lines = f.readlines()[1:]

    lines = [x for x in lines if x.startswith('2019')]
    # state is index 1
    # AVG_READING_8_SCORE is the last column
    # AVG_MATH_8_SCORE is the -3rd column
    x_values = [x.split(',')[1].replace('_', ' ') for x in lines]

    # convert to abbrev
    x_values = [STATE_ABBR[x] for x in x_values]
    y_values_reading = [int(float(x.split(',')[-1].strip())) for x in lines]
    y_values_math = [int(float(x.split(',')[-3].strip())) for x in lines]

    # sort by math
    all_values = [(x[0], x[1], x[2]) for x in zip(x_values, y_values_reading,
                                                  y_values_math)]
    all_values = sorted(all_values, key=lambda x: x[2], reverse=True)

    x_values = [x[0] for x in all_values]
    y_values_reading = [x[1] for x in all_values]
    y_values_math = [x[2] for x in all_values]

    x_pos_math = [i for i in range(len(x_values))]

    width = 0.3
    plt.bar(np.arange(len(y_values_math)), y_values_math,
            width=width, color='green')
    plt.bar(np.arange(len(y_values_reading)) + width, y_values_reading,
            width=width, color='orange')
    # plt.bar(x_pos_reading, y_values_reading, color='orange')
    plt.xlabel("State")
    plt.ylabel("Average 8th Grade Math and Reading Scores")
    plt.title("Average 8th Grade Math and Reading Scores for U.S. States")

    plt.xticks(x_pos_math, x_values, rotation=90)
    # scale the y-axis so the bars aren't so huge
    ax.set(ylim=[min(y_values_math + y_values_reading) - 5,
                 max(y_values_math + y_values_reading)])

    # set the legend
    plt.legend(['Math', 'Reading'], loc=1)
    return plt

if __name__ == "__main__":
    main()